#include "z3-api/c++/z3++.h"
#include <vector>
#include <string>
#include <iostream>

int main() {
    z3::context ctx;
    z3::solver solver(ctx);

    z3::expr x_1 = ctx.int_const("x_1");
    z3::expr x_2 = ctx.int_const("x_2");
    z3::expr x_3 = ctx.int_const("x_3");
    z3::expr x_4 = ctx.int_const("x_4");
    z3::expr x_5 = ctx.int_const("x_5");
    z3::expr x_6 = ctx.int_const("x_6");
    z3::expr x_7 = ctx.int_const("x_7");
    z3::expr x_8 = ctx.int_const("x_8");
    z3::expr x_9 = ctx.int_const("x_9");

    z3::expr sum = ctx.int_const("sum");

    z3::expr cond1 = (x_1 + x_2 + x_3) == sum;
    z3::expr cond2 = (x_4 + x_5 + x_6) == sum;
    z3::expr cond3 = (x_7 + x_8 + x_9) == sum;

    z3::expr cond4 = (x_1 + x_4 + x_7) == sum;
    z3::expr cond5 = (x_2 + x_5 + x_8) == sum;
    z3::expr cond6 = (x_3 + x_6 + x_9) == sum;

    z3::expr cond7 = (x_1 + x_5 + x_9) == sum;
    z3::expr cond8 = (x_3 + x_5 + x_7) == sum;

    solver.add(x_1 > 0);
    solver.add(x_2 > 0);
    solver.add(x_3 > 0);
    solver.add(x_4 > 0);
    solver.add(x_5 > 0);
    solver.add(x_6 > 0);
    solver.add(x_7 > 0);
    solver.add(x_8 > 0);
    solver.add(x_9 > 0);

    z3::expr_vector vec(ctx);

    vec.push_back(x_1);
    vec.push_back(x_2);
    vec.push_back(x_3);
    vec.push_back(x_4);
    vec.push_back(x_5);
    vec.push_back(x_6);
    vec.push_back(x_7);
    vec.push_back(x_8);
    vec.push_back(x_9);

    solver.add(z3::distinct(vec));

    solver.add(cond1);
    solver.add(cond2);
    solver.add(cond3);
    solver.add(cond4);
    solver.add(cond5);
    solver.add(cond6);
    solver.add(cond7);
    solver.add(cond8);

    solver.add(sum == 42);

    z3::check_result res = solver.check();
    if (res == z3::check_result::sat)
    {
        z3::model model = solver.get_model();
        
        std::cout << model.eval(x_1) << " " << model.eval(x_2) << " " << model.eval(x_3) << std::endl;
        std::cout << model.eval(x_4) << " " << model.eval(x_5) << " " << model.eval(x_6) << std::endl;
        std::cout << model.eval(x_7) << " " << model.eval(x_8) << " " << model.eval(x_9) << std::endl;
    } else
    {
        std::cout << "impossible" << std::endl;
    }

    return 0;
}
