#include "z3-api/c++/z3++.h"
#include <vector>
#include <string>
#include <iostream>
#include <cmath>

int main() {
    z3::context ctx;
    z3::solver solver(ctx);

    z3::expr x_pos = ctx.int_const("x");
    z3::expr y_pos = ctx.int_const("y");

    z3::expr in_field_x = x_pos >= 1 && x_pos <= 11;
    z3::expr in_field_y = y_pos >= 1 && y_pos <= 5;

    solver.add(in_field_x);
    solver.add(in_field_y);

    const int RED_X = 10;
    const int RED_Y = 3;

    const int BLUE_X = 2;
    const int BLUE_Y = 3;

    const int GREEN_X = 6;
    const int GREEN_Y = 3;

    solver.add(!(x_pos == RED_X && y_pos == RED_Y));
    solver.add(!(x_pos == BLUE_X && y_pos == BLUE_Y));
    solver.add(!(x_pos == GREEN_X && y_pos == GREEN_Y));

    z3::expr dist_red = (x_pos - RED_X) * (x_pos - RED_X) +
                        (y_pos - RED_Y) * (y_pos - RED_Y);

    z3::expr dist_blue = (x_pos - BLUE_X) * (x_pos - BLUE_X) +
                         (y_pos - BLUE_Y) * (y_pos - BLUE_Y);

    z3::expr dist_green = (x_pos - GREEN_X) * (x_pos - GREEN_X) +
                          (y_pos - GREEN_Y) * (y_pos - GREEN_Y);

    z3::expr dont_fall1 = (x_pos >= 3) || (y_pos >= 2);
    z3::expr dont_fall2 = (x_pos <= 9) || (y_pos >= 2);
    z3::expr dont_fall3 = (x_pos >= 3) || (y_pos <= 4);
    z3::expr dont_fall4 = (x_pos <= 9) || (y_pos <= 4);

    solver.add(dont_fall1);
    solver.add(dont_fall2);
    solver.add(dont_fall3);
    solver.add(dont_fall4);

    solver.add(dist_green < dist_red);
    solver.add(dist_green < dist_blue);
    solver.add(dist_red < dist_blue);

    z3::check_result res = solver.check();
    if (res == z3::check_result::sat)
    {
        z3::model model = solver.get_model();
        
        std::cout << "x:" << model.eval(x_pos) << " y:" << model.eval(y_pos) << std::endl;
        std::cout << "distance green: " << std::sqrt(model.eval(dist_green).get_numeral_int64()) << std::endl;
        std::cout << "distance red:   " << std::sqrt(model.eval(dist_red).get_numeral_int64()) << std::endl;
        std::cout << "distance blue:  " << std::sqrt(model.eval(dist_blue).get_numeral_int64()) << std::endl;
    } else
    {
        std::cout << "impossible" << std::endl;
    }

    return 0;
}
